import { StoreType } from '@/redux/store';
import { act, render, screen } from '@testing-library/react';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import {
  BACKGROUNDS_MOCK,
  BACKGROUND_INITIAL_STATE_MOCK,
} from '@/redux/backgrounds/background.mock';
import { initialMapStateFixture } from '@/redux/map/map.fixtures';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { OVERLAY_BIO_ENTITY_INITIAL_STATE_MOCK } from '@/redux/overlayBioEntity/overlayBioEntity.mock';
import { HttpStatusCode } from 'axios';
import { overlayBioEntityFixture } from '@/models/fixtures/overlayBioEntityFixture';
import { apiPath } from '@/redux/apiPath';
import { CORE_PD_MODEL_MOCK } from '@/models/mocks/modelsMock';
import { MODELS_INITIAL_STATE_MOCK } from '@/redux/models/models.mock';
import { parseOverlayBioEntityToOlRenderingFormat } from '@/redux/overlayBioEntity/overlayBioEntity.utils';
import { OverlayListItem } from './OverlayListItem.component';

const mockedAxiosNewClient = mockNetworkNewAPIResponse();
const DEFAULT_BACKGROUND_ID = 0;
const EMPTY_BACKGROUND_ID = 15;

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <OverlayListItem name="Ageing brain" overlayId={21} />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('OverlayListItem - component', () => {
  it('should render component with correct properties', () => {
    renderComponent();

    expect(screen.getByText('Ageing brain')).toBeInTheDocument();
    expect(screen.getByRole('button', { name: 'View' })).toBeInTheDocument();
    expect(screen.getByRole('button', { name: 'Download' })).toBeInTheDocument();
  });

  describe('view overlays', () => {
    it('should trigger view overlays on view button click and switch background to Empty if available', async () => {
      const OVERLAY_ID = 21;
      const MODEL_ID = 5053;
      const { store } = renderComponent({
        map: initialMapStateFixture,
        backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
        overlayBioEntity: OVERLAY_BIO_ENTITY_INITIAL_STATE_MOCK,
        models: { ...MODELS_INITIAL_STATE_MOCK, data: [CORE_PD_MODEL_MOCK] },
      });
      mockedAxiosNewClient
        .onGet(apiPath.getOverlayBioEntity({ overlayId: OVERLAY_ID, modelId: MODEL_ID }))
        .reply(HttpStatusCode.Ok, overlayBioEntityFixture);

      expect(store.getState().map.data.backgroundId).toBe(DEFAULT_BACKGROUND_ID);

      const ViewButton = screen.getByRole('button', { name: 'View' });
      await act(() => {
        ViewButton.click();
      });

      expect(store.getState().map.data.backgroundId).toBe(EMPTY_BACKGROUND_ID);
      expect(store.getState().overlayBioEntity.data).toEqual({
        [OVERLAY_ID]: {
          [MODEL_ID]: parseOverlayBioEntityToOlRenderingFormat(overlayBioEntityFixture, OVERLAY_ID),
        },
      });
    });
    it('should disable overlay on view button click if overlay is active', async () => {
      const OVERLAY_ID = 21;
      const { store } = renderComponent({
        map: {
          ...initialMapStateFixture,
          data: { ...initialMapStateFixture.data, backgroundId: EMPTY_BACKGROUND_ID },
        },
        backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
        overlayBioEntity: { ...OVERLAY_BIO_ENTITY_INITIAL_STATE_MOCK, overlaysId: [OVERLAY_ID] },
        models: { ...MODELS_INITIAL_STATE_MOCK, data: [CORE_PD_MODEL_MOCK] },
      });

      const ViewButton = screen.getByRole('button', { name: 'Disable' });
      await act(() => {
        ViewButton.click();
      });

      expect(store.getState().overlayBioEntity.data).toEqual([]);
      expect(store.getState().overlayBioEntity.overlaysId).toEqual([]);
    });
  });

  // TODO implement when connecting logic to component
  it.skip('should trigger download overlay to PC on download button click', () => {});
});
